/*  Thanks to https://lastminuteengineers.com and http://arduiniana.org/libraries/tinygpsplus/ for clear explanations!
    Hardware: NEO-6M GPS; ESP8266; OLED 0.96 Inch.

    Connect pin TX of GPS to D6; pin RX of GPS to D7; Oled display SDA to D2 and SCL to D1.

*/
/////////////////////////////////////////////////////////////////////////////////////////////////////
#include <TinyGPS++.h>
#include <SoftwareSerial.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

// ESP8266 pins to use for software serial
int RXPin = 12; //GPS TX pin to D6 of ESP8266
int TXPin = 13; //GPS RX pin to D7 of ESP8266

int GPSBaud = 9600;

// Create a TinyGPS++ object
TinyGPSPlus gps;

// Create a software serial port
SoftwareSerial ss(RXPin, TXPin);

void setup()
{
  // Start the Arduino hardware serial port at 9600 baud
  Serial.begin(9600);

  // Start the software serial port at the GPS's default baud
  ss.begin(GPSBaud);

  // initialize with the I2C addr 0x3C
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
}

void loop()
{
  // This sketch displays information every time a new sentence is correctly encoded.
  while (ss.available() > 0)
    if (gps.encode(ss.read()))
      displayInfo();

  // If 5000 milliseconds pass and there are no characters coming in
  // over the software serial port, show a "No GPS detected" error
  if (millis() > 5000 && gps.charsProcessed() < 10)
  {
    Serial.println(F("No GPS detected"));
    while (true);
  }
}

void displayInfo()
{
  Serial.println(F("Location: "));
  if (gps.location.isValid())
  {
    Serial.print("Latitude: ");
    Serial.println(gps.location.lat(), 6);
    Serial.print("Longitude: ");
    Serial.println(gps.location.lng(), 6);

    display.clearDisplay();

    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(0, 0);
    display.print("Latitude: ");
    display.println(gps.location.lat(), 6);
    display.setCursor(0, 11);
    display.print("Longitude: ");
    display.println(gps.location.lng(), 6);
    display.setCursor(0, 22);
    display.print("Altitude: ");
    display.println(gps.altitude.meters());
    display.setCursor(0, 33);
    display.print("Satellites in view:");
    display.println(gps.satellites.value());

    display.setCursor(0, 44);
    display.print("Date: ");
    display.print(gps.date.day());
    display.print("/");
    display.print(gps.date.month());
    display.print("/");
    display.print(gps.date.year());

    display.setCursor(0, 56);
    display.print("Time: ");
    display.print(gps.time.hour());
    display.print(":");
    display.print(gps.time.minute());
    display.print(":");
    display.print(gps.time.second());

    display.display();

  }
  else
  {
    Serial.print(F("INVALID"));
  }
  Serial.print(F("Date: "));
  if (gps.date.isValid())
  {
    Serial.print(gps.date.day());
    Serial.print(F("/"));
    Serial.print(gps.date.month());
    Serial.print(F("/"));
    Serial.println(gps.date.year());

  }
  else
  {
    Serial.print(F("INVALID"));
  }

  Serial.print(F("Time "));
  if (gps.time.isValid())
  {
    if (gps.time.hour() < 10) Serial.print(F("0"));
    Serial.print(gps.time.hour());
    Serial.print(F(":"));
    if (gps.time.minute() < 10) Serial.print(F("0"));
    Serial.print(gps.time.minute());
    Serial.print(F(":"));
    if (gps.time.second() < 10) Serial.print(F("0"));
    Serial.print(gps.time.second());
    Serial.print(F(":"));
    if (gps.time.centisecond() < 10) Serial.print(F("0"));
    Serial.println(gps.time.centisecond());

  }
  else
  {
    Serial.print(F("INVALID"));
  }

  if (gps.altitude.isUpdated())
  {
    Serial.print(F("ALT="));
    Serial.print(gps.altitude.meters());
  }
  else
  {
    Serial.println("Not Available");
  }
  if (gps.satellites.isUpdated())
  {
    Serial.print(F("SATS="));
    Serial.println(gps.satellites.value());
  }
  else
  {
    Serial.println("Not Available");
  }

  Serial.println();
  //delay(1000);

}
